@@ -8,7 +8,7 @@ use List::MoreUtils;
 
 use Polycom::Contact;
 
-our $VERSION = 0.02;
+our $VERSION = 0.03;
 
 ######################################
 # Overloaded Operators
@@ -70,6 +70,16 @@ sub new
             my ($bw) = $str =~ /<bw>(.*?)<\/bw>/s;
             my ($bb) = $str =~ /<bb>(.*?)<\/bb>/s;
 
+            foreach ($fn, $ln, $ct, $sd, $lb, $rt, $dc, $ar, $ad, $bw, $bb)
+            {
+                next if !defined;
+                s/&amp;/&/g;
+                s/&quot;/"/g;
+                s/&apos;/'/g;
+                s/&lt;/</g;
+                s/&gt;/>/g;
+            }
+
             push @contacts,
                 Polycom::Contact->new(
                 first_name     => $fn,
@@ -143,21 +153,32 @@ sub to_xml
     my ($self) = @_;
 
     my $xml = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>';
-    $xml .= "<directory>\n <item_list>\n";
+    $xml .= "\n<directory>\n <item_list>\n";
     foreach my $c ($self->all)
     {
+        my %f = %{$c};
+        foreach my $key (keys %f)
+        {
+            next if !defined $f{$key};
+            $f{$key} =~ s/&/&amp;/g;
+            $f{$key} =~ s/"/&quot;/g;
+            $f{$key} =~ s/'/&apos;/g;
+            $f{$key} =~ s/</&lt;/g;
+            $f{$key} =~ s/>/&gt;/g;
+        }
+
         $xml .= "  <item>\n";
-        $xml .= "   <fn>$c->{first_name}</fn>\n" if defined $c->{first_name};
-        $xml .= "   <ln>$c->{last_name}</ln>\n" if defined $c->{last_name};
-        $xml .= "   <ct>$c->{contact}</ct>\n" if defined $c->{contact};
-        $xml .= "   <sd>$c->{speed_index}</sd>\n" if defined $c->{speed_index};
-        $xml .= "   <lb>$c->{label}</lb>\n" if defined $c->{label};
-        $xml .= "   <rt>$c->{ring_type}</rt>\n" if defined $c->{ring_type};
-        $xml .= "   <dc>$c->{divert}</dc>\n" if defined $c->{divert};
-        $xml .= "   <ar>$c->{auto_reject}</ar>\n" if defined $c->{auto_reject};
-        $xml .= "   <ad>$c->{auto_divert}</ad>\n" if defined $c->{auto_divert};
-        $xml .= "   <bw>$c->{buddy_watching}</bw>\n" if defined $c->{buddy_watching};
-        $xml .= "   <bb>$c->{buddy_block}</bb>\n" if defined $c->{buddy_block};
+        $xml .= "   <fn>$f{first_name}</fn>\n" if defined $f{first_name};
+        $xml .= "   <ln>$f{last_name}</ln>\n" if defined $f{last_name};
+        $xml .= "   <ct>$f{contact}</ct>\n" if defined $f{contact};
+        $xml .= "   <sd>$f{speed_index}</sd>\n" if defined $f{speed_index};
+        $xml .= "   <lb>$f{label}</lb>\n" if defined $f{label};
+        $xml .= "   <rt>$f{ring_type}</rt>\n" if defined $f{ring_type};
+        $xml .= "   <dc>$f{divert}</dc>\n" if defined $f{divert};
+        $xml .= "   <ar>$f{auto_reject}</ar>\n" if defined $f{auto_reject};
+        $xml .= "   <ad>$f{auto_divert}</ad>\n" if defined $f{auto_divert};
+        $xml .= "   <bw>$f{buddy_watching}</bw>\n" if defined $f{buddy_watching};
+        $xml .= "   <bb>$f{buddy_block}</bb>\n" if defined $f{buddy_block};
         $xml .= "  </item>\n";
     }
     $xml .= " </item_list>\n</directory>";
@@ -168,6 +189,10 @@ sub to_xml
 sub save
 {
     my ($self, $filename) = @_;
+    if (!defined $filename || $filename eq '')
+    {
+        $filename = '000000000000-directory.xml';
+    }
 
     my $fh = IO::File->new($filename, '>');
     $fh->binmode(':utf8');
@@ -231,7 +256,7 @@ sub equals
 
 =head1 NAME
 
-Polycom::Contact::Directory - Module for parsing, modifying, and creating Polycom VoIP phone local contact directory files.
+Polycom::Contact::Directory - Parser for Polycom VoIP phone local contact directory files.
 
 =head1 SYNOPSIS
 
@@ -263,37 +288,49 @@ Polycom::Contact::Directory - Module for parsing, modifying, and creating Polyco
 
 =head1 DESCRIPTION
 
-This module parses Polycom VoIP phone local contact directory files, and can be used to read, modify, or create local contact directory files.
+Polycom SoundPoint IP, SoundStation IP, and VVX VoIP phones maintain a local contact directory that is stored on their configured boot server. Upon boot-up, each phone looks for a file named I<<Ethernet address>>-directory.xml on their boot server, and if found, downloads and parses the file to build up its local contact directory. In addition to basic contact information such first/last name and phone number, each contact in the local contact directory can also include information about speed-dialing, distinctive ring tones, presence, and instant messaging.
 
-=head2
+Each I<<Ethernet address>>-directory.xml contains fairly straightforward XML:
 
-=over 4
+  <?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
+  <directory>
+   <item_list>
+    <item>
+     <ln>Doe</ln>
+     <fn>John</fn>
+     <ct>1001</ct>
+     <sd>1</sd>
+     <rt>1</rt>
+     <dc/>
+     <ad>0</ad>
+     <ar>0</ar>
+     <bw>0</bw>
+     <bb>0</bb>
+    </item>
+      ...
+   </item_list>
+  </directory>
 
-=item I<Polycom::Contact::Directory>->new()
+This module parses Polycom VoIP phone local contact directory files, and can be used to read, modify, or create local contact directory files. It also provides a C<is_valid> method that can be used to perform some basic integrity checks on contact directory files.
+
+For more information about administering the Local Contact Directory on Polycom SoundPoint IP phones, see the "I<SoundPoint IP, SoundStation IP and Polycom VVX Administrator's Guide>" at L<http://www.polycom.com/support/voice/soundpoint_ip/soundpoint_ip670.html>.
+
+=head1 CONSTRUCTOR
+
+=head2 new ( $filename | $file_handle | $xml_contents )
 
   # Create a new empty directory
   my $dir = Polycom::Contact::Directory->new();
 
   # Load a directory from a filename or file handle
   my $dir2 = Polycom::Contact::Directory->new('directory.xml');
-  my $dir3 = Polycom::Contact::Directory->new(\*FILEHANDLE);
+  my $dir3 = Polycom::Contact::Directory->new($fh);
 
-If you have already slurped the contents of a contact directory file into a scalar, you can also pass that scalar to I<new()> to parse those XML contents:
+If you have already slurped the contents of a contact directory file into a scalar, you can also pass that scalar to C<new> to parse those XML contents.
 
-  my $xml = <<"END_XML";
-    <directory>
-     <item_list>
-      <item>
-       <fn>Bob</fn>
-       <ct>1234</ct>
-      </item>
-     </item_list>
-    </directory>
-  END_XML
+=head1 METHODS
 
-  my $dir = Polycom::Contact::Directory->new($xml);
-
-=item I<$dir>->insert(@contacts)
+=head2 insert ( @contacts )
 
   $dir->insert(
     {   first_name => 'Jenny',
@@ -312,7 +349,7 @@ If you have already slurped the contents of a contact directory file into a scal
 
 Adds the specified I<@contacts> contacts, if any, to the directory. I<@contacts> may be an array of hash references containing keys like "first_name", "last_name", and "contact", or it can be an array of C<Polycom::Contact> objects.
 
-=item I<$directory>->all
+=head2 all
 
   my @contacts = $dir->all;
   foreach my $contact (@contacts)
@@ -322,13 +359,13 @@ Adds the specified I<@contacts> contacts, if any, to the directory. I<@contacts>
 
 Returns an array of all of the C<Polycom::Contact> objects in the contact directory.
 
-=item I<$directory>->count
+=head2 count
 
   my $num_contacts = $dir->count;
 
 Returns the number of contacts in the directory.
 
-=item I<$directory>->equals($directory2)
+=head2 equals ( $directory2 )
 
   if ($dir1->equals($dir2))
   {
@@ -344,44 +381,50 @@ Because the I<==> and I<!=> operators have also been overloaded for both C<Polyc
     print "The contact directories are equal\n";
   }
 
-=item I<$directory>->is_valid
+=head2 is_valid
 
   if (!$dir->is_valid)
   {
     print "$dir is invalid.\n";
   }
 
-Returns true if each contact has a contact number, there are no duplicate contact numbers, and there are no duplicate speed index numbers. Otherwise, it returns false.
+Returns true if each contact is valid (e.g. has a contact number, name is < 40 bytes long, etc), there are no duplicate contact numbers, and there are no duplicate speed index numbers. Otherwise, it returns false.
 
-=item I<$directory>->save($filename_or_file_handle)
+=head2 save ( $filename )
 
   $dir->save('0004f21acabf-directory.xml');
   # or
-  $dir->save(\*FILEHANDLE);
+  $dir->save()
 
 Writes the contents of the contact directory object to the specified file such that a phone should be able to read those contacts from the file if the file is placed on the phone's boot server.
 
-=item I<$directory>->search($condition)
+If I<$filename> is not specified, the default filename used is "000000000000-directory.xml", which is the filename the phones look for if a directory file whose name contains their MAC address is not found.
+
+=head2 search ( $condition )
 
   my @smiths = $dir->search({ last_name => 'Smith' });
 
 Returns an array of the contacts that match the specified condition. I<$condition> must be a hash reference whose keys are field names of C<Polycom::Contact> fields (e.g. first_name, last_name, contact, ring_type, etc). All of the specified conditions must hold in order for a contact to be present in the array returned.
 
-=item I<$directory>->to_xml()
+=head2 to_xml
 
   my $xml = $directory->to_xml;
 
-Returns the XML representation of the contact directory. It is exactly this XML representation that the I<save()> method writes to the local contact directory file.
-
-=back
+Returns the XML representation of the contact directory. It is exactly this XML representation that the C<save> method writes to the local contact directory file.
 
 =head1 SEE ALSO
 
-Polycom::Contact - Represents a contact in the local contact directory. Each Polycom::Contact::Directory contains zero or more Polycom::Contact objects.
+=over
+
+=item C<Polycom::Contact> - A contact in the local contact directory. Each C<Polycom::Contact::Directory> object contains zero or more C<Polycom::Contact> objects.
+
+=item I<SoundPoint IP Admin Guide> - L<http://www.polycom.com/global/documents/support/setup_maintenance/products/voice/spip_ssip_vvx_Admin_Guide_SIP_3_2_2_eng.pdf>
+
+=back
 
 =head1 AUTHOR
 
-Zachary Blair, E<lt>zachary.blair@polycom.comE<gt>
+Zachary Blair, E<lt>zblair@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
